#include <QtSvg/QSvgRenderer>
#include <QtGui/QPainter>

#include "qtsvgbutton.h"

QtSvgButton::QtSvgButton(QWidget * parent)
    : QAbstractButton(parent)
{
    // trigger repaint on enter and leave event
    setAttribute(Qt::WA_Hover, true);
}

QtSvgButton::~QtSvgButton()
{
}

void QtSvgButton::setSkin(const QString& skin)
{
    m_skin = skin;
    QString base = ":/svgbutton/" + skin + '/';

    // create null size and null pictures
    m_defaultSize = QSize();
    m_normal = QPicture();
    m_hovered = QPicture();
    m_pressed = QPicture();

    QSvgRenderer renderer;
    QPainter painter;

    if (renderer.load(base + "normal.svg")) {
        m_defaultSize = renderer.defaultSize();
        painter.begin(&m_normal);
        renderer.render(&painter, QRectF(0.0, 0.0, 1.0, 1.0));
        painter.end();
    }

    if (renderer.load(base + "pressed.svg")) {
        painter.begin(&m_pressed);
        renderer.render(&painter, QRectF(0.0, 0.0, 1.0, 1.0));
        painter.end();
    }

    if (renderer.load(base + "hovered.svg")) {
        painter.begin(&m_hovered);
        renderer.render(&painter, QRectF(0.0, 0.0, 1.0, 1.0));
        painter.end();
    }

    // update geometry for new sizeHint and repaint
    updateGeometry();
    update();
}

QString QtSvgButton::skin() const
{
    return m_skin;
}

QRect QtSvgButton::buttonRect() const
{

    // Keep aspect ratio:
    QSize buttonSize = m_defaultSize;
    buttonSize.scale(rect().size(), Qt::KeepAspectRatio);

    // Keep the button aligned horizontally centered (like the text):
    int buttonPosX = (rect().width() - buttonSize.width()) / 2;
    if (buttonPosX < 0) buttonPosX = 0;
    buttonPosX += rect().left();

    QPoint buttonPos = QPoint(buttonPosX, 0);
    if(skin() == "MyBerylSquare")
        return rect();
    return QRect(buttonPos, buttonSize);
}

void QtSvgButton::paintEvent(QPaintEvent * event)
{
    Q_UNUSED(event);

    QPainter painter(this);
    QRect rect = painter.viewport();
    painter.scale(buttonRect().width(), buttonRect().height());
    if (isDown()) {
        m_pressed.play(&painter);
        if(skin() == "MyBerylSquare") {
            QLinearGradient linearGrad(rect.topLeft(), rect.bottomRight());
            linearGrad.setColorAt(0.0, Qt::darkGray);
            linearGrad.setColorAt(1.0, Qt::black);
            drawText(linearGrad, rect);
        }
    } else if (underMouse()) {
        m_hovered.play(&painter);
        if(skin() == "MyBerylSquare") {
            QLinearGradient linearGrad(rect.topLeft(), rect.bottomRight());
            linearGrad.setColorAt(0.0, Qt::darkGreen);
            linearGrad.setColorAt(1.0, Qt::yellow);
            drawText(linearGrad, rect);
        }
    } else {
        m_normal.play(&painter);
        if(skin() == "MyBerylSquare") {
            QLinearGradient linearGrad(rect.topLeft(), rect.bottomRight());
            linearGrad.setColorAt(0.0, Qt::darkGray);
            linearGrad.setColorAt(1.0, Qt::black);
            drawText(linearGrad, rect);
        }
    }
}

void QtSvgButton::drawText(const QLinearGradient &lgrad, QRect viewportRect)
{
    QPainter painter1(this);
    painter1.setViewport(viewportRect);

    static const float xkof = -20.0 / viewportRect.width();
    static const float ykof = -18.0 / viewportRect.height();
    viewportRect.adjust(1, 1,(int)(xkof*viewportRect.width()), (int)(ykof*viewportRect.height()));

    QBrush myBrush(lgrad);
    QPen myPen(myBrush,1);
    QFont myFont;
    myFont.setPixelSize(viewportRect.height()/2);
    QFontMetrics fontmetrics(myFont);

    QPointF baseline(viewportRect.center());
    baseline += QPoint(-fontmetrics.width(text())/2, (int)(fontmetrics.height()/2*0.61));
    QPainterPath myPath;
    myPath.addText(baseline, myFont, text());

    painter1.setPen(myPen);
    painter1.setBrush(myBrush);
    painter1.setClipRect(viewportRect);
    painter1.setRenderHint(QPainter::Antialiasing, true);

    painter1.drawPath(myPath);
}

QSize QtSvgButton::sizeHint() const
{
    if (!m_defaultSize.isEmpty()) {
        return m_defaultSize;
    } else {
        return QSize(100, 24);
    }
}
